//
// shader_dynamicSky2clouds.fx
// Author: Ren712/AngerMAN
//

//------------------------------------------------------------------------------------------
//-- Settings
//------------------------------------------------------------------------------------------
float gDayTime = 0;
float mMoonLightInt = 1;
bool gIsInWater = false;
float3 gScale = float3(1, 1, 1);
float gRescale = 1;
float2x3 gSunColor = {0,0,0,0,0,0};
float3 gRotate = float3(0,0,0);
float3 mRotate = float3(0,0,0);
float gCloudSpeed = 0.0;
float2 gStratosFade = float2(14000, 10000);
float gAlphaMult = 1;
float gBottCloudSpread = 500;
bool gHorizonBlending = true;

float GameTime = 24;
float currentWeather = 0;
float3 WeatherAndTime;
float3 ColorSkyTop = { 0.5, 0.5, 0.5 } ;
float3 ColorSkyBottom = { 0.5, 0.5, 0.5 } ;
texture texBlueN;
texture texclouds1;
texture texclouds2;
texture texclouds3;
texture sStars;
texture sMoon1;
//------------------------------------------------------------------------------------------
//-- Include some common things
//------------------------------------------------------------------------------------------
float4x4 gWorld : WORLD;
float4x4 gView : VIEW;

float4x4 gViewInverse : VIEWINVERSE;
float4x4 gWorldInverseTranspose : WORLDINVERSETRANSPOSE;
float4x4 gViewInverseTranspose : VIEWINVERSETRANSPOSE;

float4x4 gProjection : PROJECTION;
float4x4 gWorldViewProjection : WORLDVIEWPROJECTION;
float3 gCameraPosition : CAMERAPOSITION;
float3 gCameraDirection : CAMERADIRECTION;
matrix gProjectionMainScene : PROJECTION_MAIN_SCENE;
int gFogEnable < string renderState="FOGENABLE"; >;
float4 gFogColor < string renderState="FOGCOLOR"; >;
float gFogStart < string renderState="FOGSTART"; >;
float gTime : TIME;
int CUSTOMFLAGS < string skipUnusedParameters = "yes"; >;

//------------------------------------------------------------------------------------------
//-- Sampler for the main texture (needed for pixel shaders)
//------------------------------------------------------------------------------------------
sampler2D SamplerNoise = sampler_state
{
   Texture   = <texBlueN>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

sampler2D SamplerCl1 = sampler_state
{
   Texture   = <texclouds1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

sampler2D SamplerCl2 = sampler_state
{
   Texture   = <texclouds2>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

sampler2D SamplerCl3 = sampler_state
{
   Texture   = <texclouds3>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

sampler2D SamplerStars = sampler_state
{
   Texture   = <sStars>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU = Wrap;
    AddressV = Wrap;
    AddressW = Wrap;
};

sampler2D SamplerMoon = sampler_state
{
   Texture = <sMoon1>;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = NONE;
    AddressU = Clamp;
    AddressV = Clamp;
};

//------------------------------------------------------------------------------------------
//-- Structure of data sent to the vertex shader
//------------------------------------------------------------------------------------------
 struct VSInput
{
    float4 Position : POSITION; 
    float3 TexCoord : TEXCOORD0;
};

//------------------------------------------------------------------------------------------
//-- Structure of data sent to the pixel shader ( from the vertex shader )
//------------------------------------------------------------------------------------------
struct PSInput
{
    float4 Position : POSITION; 
    float3 TexCoord0 : TEXCOORD0;
    float3 TexCoord1 : TEXCOORD1;
    float3 TexCoord2 : TEXCOORD2;
    float3 TexCoord3 : TEXCOORD3;
    float3 NightFade : TEXCOORD4;
    float3 LimitDot : TEXCOORD5;
    float4 WorldPos : TEXCOORD6;
    float3 PositionVS : TEXCOORD7;
};

//------------------------------------------------------------------------------------------
//-- eulerRotate
//------------------------------------------------------------------------------------------
float3x3 eulerRotate(float3 Rotate)
{
    float cosX,sinX;
    float cosY,sinY;
    float cosZ,sinZ;
	
    sincos(Rotate.x ,sinX,cosX);
    sincos(-Rotate.y ,sinY,cosY);
    sincos(Rotate.z ,sinZ,cosZ);

    return float3x3(
		cosY * cosZ + sinX * sinY * sinZ, -cosX * sinZ,  sinX * cosY * sinZ - sinY * cosZ,
		cosY * sinZ - sinX * sinY * cosZ,  cosX * cosZ, -sinY * sinZ - sinX * cosY * cosZ,
		cosX * sinY,                       sinX,         cosX * cosY
	);
}

//------------------------------------------------------------------------------------------
// MTAUnlerp
//------------------------------------------------------------------------------------------
float MTAUnlerp( float from, float to, float pos )
{
    if ( from == to )
        return 1.0;
    else
        return ( pos - from ) / ( to - from );
}

//------------------------------------------------------------------------------------------
//-- VertexShader
//------------------------------------------------------------------------------------------
PSInput VertexShaderSB(VSInput VS)
{
    PSInput PS = (PSInput)0;	
    PS.PositionVS = VS.Position.xyz;
    float farClip = gProjectionMainScene[3][2] / (1 - gProjectionMainScene[2][2]);
    if ((gFogEnable) && (gIsInWater)) farClip = min(gFogStart, farClip);
    VS.Position.xyz *= gScale * max(230, farClip) * 0.31 * gRescale;
    float4x4 sWorld = gWorld; sWorld[3].xyz = float3(0,0,0);
    float4 mulWorld = mul(VS.Position, sWorld);
    float4x4 sView = gView; sView[3].xyz = float3(0,0,0);
    float4 mulView = mul( mulWorld, sView);
    PS.Position = mul(mulView, gProjection);
    PS.TexCoord1 = normalize(mul(eulerRotate(gRotate), VS.Position.xyz));
    PS.TexCoord2 = normalize(mul(eulerRotate(mRotate), VS.Position.xyz)) * 8;
    PS.WorldPos = mul(VS.Position, sWorld);
    float3 eyeVector = normalize(PS.WorldPos.xyz - 0); 		 
    PS.TexCoord3 = mul(eulerRotate(gRotate), eyeVector.xyz* float3(19.0, 19.0, 19.0));	
    float2 move = float2(gCameraPosition.x * 0.00001, gCameraPosition.z * 0.0005);
    PS.TexCoord0 = VS.TexCoord - float3(move.x, move.y, 0);
    PS.NightFade.x = saturate(MTAUnlerp(gStratosFade[1], gStratosFade[1] - 2000, gCameraPosition.z));
    PS.NightFade.y = saturate(dot( float3(0,0,-1), PS.TexCoord2));
    PS.NightFade.z = saturate(MTAUnlerp(farClip + abs(gBottCloudSpread) + 50, farClip + 50, gCameraPosition.z));
	
    if (gHorizonBlending) PS.LimitDot.x = saturate(2.2 * PS.PositionVS.z);
        else PS.LimitDot.x = 1.0;
    float topDot = dot(float3(0,0,1), PS.PositionVS.xyz);	
    PS.LimitDot.y = 1 - saturate(pow(max(0.0001, topDot), 6.4) * 0.001);
    PS.LimitDot.z = dot(float3(0,0,-1), normalize(PS.TexCoord2));	
    return PS;
}

//------------------------------------------------------------------------------------------
// Structure of color data sent to the renderer ( from the pixel shader  )
//------------------------------------------------------------------------------------------
struct Pixel
{
    float4 World : COLOR0;      // Render target #0
    float Depth : DEPTH;        // Depth target
};

//------------------------------------------------------------------------------------------
//-- PixelShader
//------------------------------------------------------------------------------------------

float SunLight(float3 worldP)
{

float3  f = lerp(float3(0.,0.,-.61), float3(.583,-.257,-.0108), smoothstep(0.,5.,GameTime));
        f = lerp(f, float3(.569,-.242,.151), smoothstep(5.,7.,GameTime));
		f = lerp(f, float3(.49,-.247,.323), smoothstep(7.,9.,GameTime));
		f = lerp(f, float3(.373,-.262,.445), smoothstep(9.,11.,GameTime));
		f = lerp(f, float3(.201,-.312,.517), smoothstep(11.,13.,GameTime));
		f = lerp(f, float3(-.00732,-.39,.503), smoothstep(13.,15.,GameTime));
		f = lerp(f, float3(-.189,-.502,.343), smoothstep(15.,17.,GameTime));
		f = lerp(f, float3(-.223,-.594,.1), smoothstep(17.,19.,GameTime));
		f = lerp(f, float3(0.,0.,-.61), smoothstep(19.,24.,GameTime));
		f = normalize(f);

   float3 v0= f.xyz;
   float3 v2= normalize(float3(-0.09, -0.94, 0.31));	
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 4.0, t0);	
   float x2 = smoothstep(4.0, 23.0, t0);
   float x3 = smoothstep(23.0, 24.0, t0);	
   
   float3 sv = lerp(v2, v0, x1);
          sv = lerp(sv, v0, x2);
          sv = lerp(sv, v2, x3);
		  
   float3 np = normalize(worldP.xyz);
   float f0 = 0.95 - dot(-sv, np);
         f0 = pow(f0, 6.07);
   float r0 = (f0*f0);
   
   return r0/40000.0;
}

float IntersectPlane(float3 pos, float3 dir)
{
	return -pos.z/dir.z;
}

float Stars(in float2 coord)
{	
    float2 uv = coord.xy;
	float2 p = (uv) / (0.05*2.0);
	float fstars = tex2D(SamplerStars, p);
	return fstars;
}

float Moon(in float2 coord)
{	
    float2 uv = coord.xy + 0.1 * 1.5;
	float2 p = (uv + 0.5) / (-3.25*2.0);
	float fmoon = tex2D(SamplerMoon, p);
	return fmoon*1.5;
}

/**/
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------

float4 CalculateSun(float3 worldp)
{
float3  f = lerp(float3(0.,0.,-.61), float3(.583,-.257,-.0108), smoothstep(0.,5.,GameTime));
        f = lerp(f, float3(.569,-.242,.151), smoothstep(5.,7.,GameTime));
		f = lerp(f, float3(.49,-.247,.323), smoothstep(7.,9.,GameTime));
		f = lerp(f, float3(.373,-.262,.445), smoothstep(9.,11.,GameTime));
		f = lerp(f, float3(.201,-.312,.517), smoothstep(11.,13.,GameTime));
		f = lerp(f, float3(-.00732,-.39,.503), smoothstep(13.,15.,GameTime));
		f = lerp(f, float3(-.189,-.502,.343), smoothstep(15.,17.,GameTime));
		f = lerp(f, float3(-.223,-.594,.1), smoothstep(17.,19.,GameTime));
		f = lerp(f, float3(0.,0.,-.61), smoothstep(19.,24.,GameTime));
		f = normalize(f);	
		
    float3 sv = f.xyz;
    float3 sv2 = normalize(float3(-0.0833, -0.946, 0.317));
	
   float t = GameTime;	
	
   float x1 = smoothstep(0.0, 4.0, t);
   float x2 = smoothstep(4.0, 5.0, t);
   float x3 = smoothstep(5.0, 6.0, t);
   float x4 = smoothstep(6.0, 7.0, t);
   float xE = smoothstep(8.0, 11.0, t);
   float x5 = smoothstep(16.0, 17.0, t);
   float x6 = smoothstep(18.0, 19.0, t);
   float x7 = smoothstep(19.0, 20.0, t);
   float xG = smoothstep(20.0, 21.0, t);  
   float xZ = smoothstep(21.0, 22.0, t);
   float x8 = smoothstep(22.0, 23.0, t);
   float x9 = smoothstep(23.0, 24.0, t);

   float3 SC0 = float3(0.9, 0.7, 0.6);   
   float3 t0 = lerp(0.0, 0.0, x1);
          t0 = lerp(t0, SC0*SC0*0.5, x2);
          t0 = lerp(t0, SC0*SC0*2.7, x3);
          t0 = lerp(t0, float3(1.0, 0.863, 0.549), x4);
          t0 = lerp(t0, float3(1.0, 0.863, 0.549), xE);
          t0 = lerp(t0, float3(1.0, 0.863, 0.549), x5);
          t0 = lerp(t0, SC0*SC0*2.0, x6);	 
          t0 = lerp(t0, SC0*SC0*0.8, x7);
		  t0 = lerp(t0, SC0*0.6, xG);
		  t0 = lerp(t0, SC0*0.4, xZ);
          t0 = lerp(t0, SC0*0.2, x8);
          t0 = lerp(t0, 0.0, x9);	

   float3 SC3 = float3(0.9, 0.7, 0.6); 		  
   float3 t3 = lerp(0.0, 0.0, x1);
          t3 = lerp(t3, SC3*SC3*0.5, x2);
          t3 = lerp(t3, SC3*SC3*2.7, x3);
          t3 = lerp(t3, float3(1.1, 0.7, 0.6)*2.0, x4);
          t3 = lerp(t3, float3(1.0, 1.0, 1.0)*2.0, xE);
          t3 = lerp(t3, float3(1.0, 1.0, 1.0)*2.0, x5);
          t3 = lerp(t3, SC3*SC3*2.7, x6);		 
          t3 = lerp(t3, SC3*SC3*0.8, x7);
		  t3 = lerp(t3, SC3*0.6, xG);
		  t3 = lerp(t3, SC3*0.4, xZ);
          t3 = lerp(t3, SC3*0.2, x8);
          t3 = lerp(t3, 0.0, x9); 	
   
   float3 wp = worldp;
   float c0 = 475.0 * 25.0;
   float c1 = 18.0;
   float c2 = 1.35;
   
   float3 np0 = normalize(wp.xyz);
   float factor = (0.01/12.0) - dot(-sv, np0);
         factor = pow(factor, c0);
   float factor1 = 0.04 - dot(-sv, np0);
         factor1 = pow(factor1, c1);	
   float factor2 = 0.6 - dot(-sv, np0);
         factor2 = pow(factor2, c2);
		 
   float factor3 = 0.04 - dot(-sv2, np0);
         factor3 = pow(factor3, c1);	
   float factor4 = 0.6 - dot(-sv2, np0);
         factor4 = pow(factor4, c2);		 
		 
   float3 f0 = factor/12.0;
   float3 f1 = (factor1*factor1)/5.0;    
   float3 f2 = (factor2*factor2)/10.0;
   
   float3 f1x = (factor3*factor3)/5.0;    
   float3 f2x = (factor4*factor4)/10.0;   
   float3 fnight = (f1x*float3(0.0863, 0.137, 0.176))+(f2x*float3(0.0, 0.0, 0.0));     
		  fnight*= smoothstep(0.0, 0.3, -np0.y);   
   
   float y1 = smoothstep(0.0, 2.0, t);
   float yZ = smoothstep(2.0, 3.0, t);   
   float y2 = smoothstep(4.0, 23.0, t);
   float y3 = smoothstep(23.0, 24.0, t);   
   
  float3 t4 = lerp(0.0, 0.0, y1);
         t4 = lerp(t4, float3(0.0392, 0.0335, 0.027), yZ);  
         t4 = lerp(t4, float3(0.0392, 0.0235, 0.027), y2);
         t4 = lerp(t4, 0.0, y3);
		 
  float3 t5 = lerp(1.0, 0.5, y1);
         t5 = lerp(t5, 0.0, yZ);  
         t5 = lerp(t5, 0.0, y2);
         t5 = lerp(t5, 1.0, y3);	 
 
   float3 wp0 = wp; //  gCameraPosition
   float3 np1 = normalize(wp0.xyz);
   float4 f3 = 1.0;
          f3.xyz = lerp(0.0, (f0*t4), smoothstep(0.0, 0.02, np1.z))+(f1*t0*0.05) +(f2*t3*0.8)+(fnight*t5*5.0); // +(f1*t0)+(f2*t3)+(fnight*t5);
		  
   float sA = f3.xyz;		  
		  f3.a = lerp(0.0, sA, smoothstep(0.0, 0.2, np1.z));
		  
		  


float wx = currentWeather;
float4 SunCurrent;

if (wx.x==0,1) SunCurrent = f3;

if (wx.x==4) SunCurrent = 0.0;
if (wx.x==7) SunCurrent = 0.0;
if (wx.x==8) SunCurrent = 0.0;
if (wx.x==9) SunCurrent = 0.0;
if (wx.x==12) SunCurrent = 0.0;
if (wx.x==15) SunCurrent = 0.0;
if (wx.x==16) SunCurrent = 0.0;	  

   return SunCurrent;
}

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------

Pixel PS_Sky(PSInput PS, float2 vPos : VPOS0)
{
    Pixel output;
	
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 4.0, t0);
   float x2 = smoothstep(4.0, 5.0, t0);
   float x3 = smoothstep(5.0, 6.0, t0);
   float x4 = smoothstep(6.0, 7.0, t0);
   float xE = smoothstep(8.0, 11.0, t0);
   float x5 = smoothstep(16.0, 17.0, t0);
   float x6 = smoothstep(18.0, 19.0, t0);
   float x7 = smoothstep(19.0, 20.0, t0);
   float xG = smoothstep(20.0, 21.0, t0);  
   float xZ = smoothstep(21.0, 22.0, t0);
   float x8 = smoothstep(22.0, 24.0, t0);
	
   float Timex = 0.005 * gTime;
   float Timex1 = 0.01 * gTime;	
	
   float3 TexCoord1 = normalize(PS.TexCoord1);
   float3 TexCoord2 = normalize(PS.TexCoord2);
			   
float3 sLightDir = normalize(float3(-.00732,-.39,.503));
		     
   float3 wpos = PS.WorldPos.xyz;  
   float2 res = PS.TexCoord0.xy;
   float3 p0 = (gViewInverseTranspose[3].xyz/gViewInverseTranspose[3].w);	
	
   float3 w1 = float3(1.0, 1.0, 10.0);
   float4 ns = float4(normalize(wpos.xyz), 1.0);    
   
   float3 ns1 = normalize(wpos.xyz);
          ns1.z = abs(ns1.z)*1.40;	

    float3 np2 = normalize(-p0+ns1);

		  
   float4 Sun = CalculateSun(wpos);	
   float Al = Sun;	 
   
   float4 cd = float4(1.0, 1.0, 1.0, 0.0);
	      //cd+= Sun*1.0;
	      cd.xyz = Sun.xyz*1.0;
		  
    float pos = frac(vPos.xy/2.0) + 0.25;
    float3 j0 = float3(0.25,-0.25,0.25) * (1.0 / (exp2(8.0) - 1.0));
     	   j0 = lerp(2.0 * j0, -2.0 * j0, pos);					  

          cd.rgb+= j0;		  
		  
	      cd.a = Sun.a;		
	      cd.a+= j0;			  
    output.World = cd;	
    output.Depth = 0.99999f;
    return output;
}
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------

Pixel PS_Stars(PSInput PS, float2 vPos : VPOS0)
{
    Pixel output;
	
	
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 4.0, t0);
   float x2 = smoothstep(4.0, 5.0, t0);
   float x3 = smoothstep(5.0, 6.0, t0);
   float x4 = smoothstep(6.0, 7.0, t0);
   float xE = smoothstep(8.0, 11.0, t0);
   float x5 = smoothstep(16.0, 17.0, t0);
   float x6 = smoothstep(18.0, 19.0, t0);
   float x7 = smoothstep(19.0, 20.0, t0);
   float xG = smoothstep(20.0, 21.0, t0);  
   float xZ = smoothstep(21.0, 22.0, t0);
   float x8 = smoothstep(22.0, 24.0, t0);
	
   float Timex = 0.005 * gTime;
   float Timex1 = 0.01 * gTime;	
	
   float3 TexCoord1 = normalize(PS.TexCoord1);
   float3 TexCoord2 = normalize(PS.TexCoord2);
			   
float3 sLightDir = normalize(float3(-.00732,-.39,.503));
		     
   float3 wpos = PS.WorldPos.xyz;  
   float2 res = PS.TexCoord0.xy;
   float3 p0 = (gViewInverseTranspose[3].xyz/gViewInverseTranspose[3].w);	

	
   float3 sc0 = SunLight(wpos);	
	
   float3 w1 = float3(1.0, 1.0, 10.0);
   float4 ns = float4(normalize(wpos.xyz), 1.0);    
   
   float3 ns1 = normalize(wpos.xyz);
          ns1.z = abs(ns1.z)*1.40;	

    float3 np2 = normalize(-p0+ns1);

   float sm1 = smoothstep(0.0, 4.0, t0),	
         sm2 = smoothstep(4.0, 5.0, t0),	 
         sm3 = smoothstep(5.0, 6.0, t0),			 
         sm4 = smoothstep(6.0, 23.0, t0),
         sm5 = smoothstep(23.0, 24.0, t0);
   float3 ti = lerp(1.0, 1.0, sm1);
          ti = lerp(ti, 0.3, sm2);   
          ti = lerp(ti, 0.0, sm3); 		  
          ti = lerp(ti, 0.0, sm4);
          ti = lerp(ti, 1.0, sm5);

   float4 s0 = Stars(np2 * 0.5 + 0.5)*3.2*1.40;
   		  s0*= smoothstep(0.0, 0.3, ns.z);
		  s0.a = min(1.0, ti*s0.a);
		  
	
   float4 cd = float4(1.0, 1.0, 1.0, 0.0);
	      cd = s0*1.3;
       // cd.a  = lerp(0.0, cd.a, smoothstep(0.0, 0.10, pow(ns.z, 1.65)));	

    float pos = frac(vPos.xy/2.0) + 0.25;
    float3 j0 = float3(0.25,-0.25,0.25) * (1.0 / (exp2(8.0) - 1.0));
     	   j0 = lerp(2.0 * j0, -2.0 * j0, pos);					  

          cd.rgb+= j0;
		
    output.World = cd;	
    output.Depth = 0.99999f;
    return output;
}

//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------

Pixel PS_Moon(PSInput PS, float2 vPos : VPOS0)
{
    Pixel output;
		
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 4.0, t0);
   float x2 = smoothstep(4.0, 5.0, t0);
   float x3 = smoothstep(5.0, 6.0, t0);
   float x4 = smoothstep(6.0, 7.0, t0);
   float xE = smoothstep(8.0, 11.0, t0);
   float x5 = smoothstep(16.0, 17.0, t0);
   float x6 = smoothstep(18.0, 19.0, t0);
   float x7 = smoothstep(19.0, 20.0, t0);
   float xG = smoothstep(20.0, 21.0, t0);  
   float xZ = smoothstep(21.0, 22.0, t0);
   float x8 = smoothstep(22.0, 24.0, t0);
	
   float Timex = 0.005 * gTime;
   float Timex1 = 0.01 * gTime;	
	
   float3 TexCoord1 = normalize(PS.TexCoord1);
   float3 TexCoord2 = normalize(PS.TexCoord2);
			   
   float3 sLightDir = normalize(float3(-.00732,-.39,.503));
		     
   float3 wpos = PS.WorldPos.xyz;  
   float2 res = PS.TexCoord0.xy;
   float3 p0 = (gViewInverseTranspose[3].xyz/gViewInverseTranspose[3].w);	

	
   float3 sc0 = SunLight(wpos);	
	
   float3 w1 = float3(1.0, 1.0, 10.0);
   float4 ns = float4(normalize(wpos.xyz), 1.0);    
   
   float3 ns0 = normalize(wpos.xzy+float3(5.0, 5.0, 0.0));
   
   float v1 = 2.89;
   float3x3 vec0 = float3x3(1, 0, 0, 0, cos(v1), sin(v1), 0, -sin(v1), cos(v1));   
		   ns0 = mul(ns0, vec0);		   
	float3 np1 = normalize(ns0);

   float sm1 = smoothstep(0.0, 4.0, t0),	
         sm2 = smoothstep(4.0, 5.0, t0),	 
         sm3 = smoothstep(5.0, 6.0, t0),			 
         sm4 = smoothstep(6.0, 23.0, t0),
         sm5 = smoothstep(23.0, 24.0, t0);
   float3 ti = lerp(1.0, 1.0, sm1);
          ti = lerp(ti, 0.3, sm2);   
          ti = lerp(ti, 0.0, sm3); 		  
          ti = lerp(ti, 0.0, sm4);
          ti = lerp(ti, 1.0, sm5);

   float4 m0 = Moon(np1*53.0)*0.78;
   		  m0*= smoothstep(0.0, 0.3, -ns.y);
		  m0.a = saturate(min(1.0, ti*m0.a));
		  
	
   float4 cd = float4(1.0, 1.0, 1.0, 0.0);
	      cd.xyz = m0.xyz*0.7;
		  
    float pos = frac(vPos.xy/2.0) + 0.25;
    float3 j0 = float3(0.25,-0.25,0.25) * (1.0 / (exp2(8.0) - 1.0));
     	   j0 = lerp(2.0 * j0, -2.0 * j0, pos);					  

          cd.rgb+= j0;		  
		  
	      cd.a = m0.a;		  
       // cd.a  = lerp(0.0, cd.a, smoothstep(0.0, 0.10, pow(ns.z, 1.65)));	
		
    output.World = cd;	
    output.Depth = 0.99999f;
    return output;
}


//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------

float noise(in float3 x)
{
    float3 p = floor(x);
    float3 f = frac(x);
	float2 uv = (p.xy+float2(37.0,17.0)*p.z) + f.xy;
	float2 rg = tex2Dlod(SamplerCl1, float4((uv.xy+0.5)/256.0,0,0)).yx;
	return lerp( rg.x, rg.y, saturate(f.z));
}

float ns(float3 p)
{
	float v;
          v = 0.5000*noise( p );
          v+= 0.2000*noise( p )*1.6;
   return v;
}


#define THICKNESS		250.0
#define ABSORPTION		1.030725
#define STEPS			54.0

#define cl_start 275.0
#define cl_end   730.0
#define cl_time (gTime * 1.0) //1


float CloudsSize = 0.06;
float CloudsSize2 = 0.3;

float CloudsL = 0.5;
float CloudsL2 = 0.5;

float CloudsClamp = 0.4;
float CloudsClamp2 = 0.3;

float CloudsmixLight = 0.3;


float cl1(float3 p, out float cloudHeight, float rc, float bnoise)
{
    float3 v0 = float3(0.05, 0.22, -0.65);	
    p += v0 * p.z;
    p.y += cl_time*20.0;

    float3 p1 = p;
	       p1.xy-= cl_time*146.0;
		   
    float3 p2 = p;
	       p2.xy+= cl_time*10.0;
		   
  	cloudHeight = saturate((p.z-cl_start)/(cl_end-cl_start));	
    float clouds1 = clamp((tex2Dlod(SamplerCl2, float4(p1.xy*CloudsSize*0.0005,0,0)).x)*CloudsL, 0.0, CloudsClamp);	
          clouds1*= smoothstep(0.0, 0.05, cloudHeight) * smoothstep(1.0, 0.5, cloudHeight);		
          clouds1 = lerp(1.0, clouds1, 0.9);
		  
		  p.xy += cl_time*25.0;		
		  
    float clouds2 = clouds1*clamp((tex2Dlod(SamplerCl3, float4(0.0005*p2.xy * CloudsSize2 * float2(0.9,1.0), 0,0)).x*CloudsL2)*0.9, 0.0, CloudsClamp2); //clouds1*
          clouds2 *= smoothstep(0.0, 0.5, cloudHeight) * smoothstep(1.0, 0.5, cloudHeight);			  

  	float sh = pow(clouds2, 0.3+0.3*smoothstep(0.1, 0.5, cloudHeight));
    	if(sh <= 0.0) return 0.0;
	
	float t = -0.15;
	float d = max(0.0, sh-0.0);
    return CloudsmixLight*saturate(5.0 * d-(ns(bnoise+p*0.05*0.1) + 0.0) + t - rc);  //- rc

}

float Density(float3 p, out float cloudHeight, float bnoise)
{
    float3 v0 = float3(0.05, 0.22, -0.65);

    p+= v0 * p.z;
    p.y += cl_time*20.0;
	
    float3 p1 = p;
	       p1.xy-= cl_time*146.0;

    float3 p2 = p;
	       p2.xy+= cl_time*10.0;	

  	cloudHeight = saturate((p.z-cl_start)/(cl_end-cl_start));
    float clouds1 = clamp((tex2Dlod(SamplerCl2, float4(p1.xy*CloudsSize*0.0005,0,0)).x)*CloudsL, 0.0, CloudsClamp);
          clouds1*= smoothstep(0.0, 0.05, cloudHeight) * smoothstep(1.0, 0.5, cloudHeight);	
          clouds1 = lerp(1.0, clouds1, 0.85);	
		  
		  p.xy += cl_time*25.0;		
    float clouds2 = clouds1*clamp((tex2Dlod(SamplerCl3, float4(0.0005*p2.xy * CloudsSize2 * float2(0.9,1.0), 0,0)).x*CloudsL2)*0.9, 0.0, CloudsClamp2); //clouds1*
          clouds2 *= smoothstep(0.0, 0.5, cloudHeight) * smoothstep(1.0, 0.5, cloudHeight);				  
		  
  	float sh = pow(clouds2, 0.3+0.3*smoothstep(0.1, 0.5, cloudHeight));
    	if(sh <= 0.0) return 0.0;

	float d = max(0.0, sh-0.2);
	float res = 0.8*saturate(5.0 * d-(ns(bnoise+p*0.05*0.62) + 0.05) + 0.15)*(ns(bnoise+p*0.05*0.62) + 0.02);		  
		  
	return res;

}

float bn(float2 vpos)
{
	float n = tex2Dlod(SamplerNoise, float4(vpos.xy*0.01, 0.0, 0.0));
   return n;
}


float dithering(float2 vpos, int l)
{
	float res = 0.0;
	for(float i = 1-l; i<= 0; i++)
	{
		float Size = exp2(i);
	    float2 coord = floor(vpos*Size) % 2.0;
		float cd = 2.0*coord.x - 4.0*coord.x*coord.y + 3.0*coord.y;
		res += exp2(2.0*(i+l))* cd;
	}
	float res1 = 4.0*exp2(2.0 * l)- 4.0;
	return res/res1 + 1.0/exp2(2.0 * l);
}


float lightRay(float3 p, float3 sundir, float ch, float r, float RayAll, float rc, float bnoise)
{
    int samples = 3;
    float step = (r*150.0)/float(samples);   
    float3 v0 = float3(0.05, 0.2, -0.65);
	
    float lighRay = 0.0;    
    p+= sundir*step*10.0;
    p+= 0.0 * p.z * float3(1,1,0);	
	
    for(int j=0; j<samples; j++)
    {
        float ch;		
        lighRay += cl1( p + sundir * 2.7 / RayAll * float(j) * step, ch, rc, bnoise);
    }  

	float res = rcp(lighRay*lighRay+0.001*5.37); 
   return res;
}


float3 tonemapping(float3 color)
{
	float3 cn0 = normalize(color.xyz),
	       ct0=color.xyz/cn0.xyz;
	       ct0=pow(ct0, 0.65);
           cn0.xyz = pow(cn0.xyz, 0.9);   
           color.xyz = ct0*cn0.xyz;  
           color.xyz*= 0.04;

    float maxcol = max(color.x, max(color.y, color.z));		 		 
    float3 r0 = 1.0-exp(-maxcol),
           r2 = 1.0-exp(-color);	

         color.xyz = lerp(float3(r2.x, r2.y, r2.z), color*float3(r0.x, r0.y, r0.z)/maxcol, 0.05);
  return color;	
}


float Luminance(in float3 color)
{
	return dot(color.rgb, float3(0.2125f, 0.7154f, 0.0721f));
}



float4 Cloud_mid(const in float3 dir, const in float3 cam_pos, const in float d0, float height, float2 vpos, float3 SunDirection)
{	
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 3.0, t0);
   float x2 = smoothstep(3.0, 4.0, t0);
   float x3 = smoothstep(4.0, 6.0, t0);
   float x4 = smoothstep(6.0, 7.0, t0);
   float xE = smoothstep(8.0, 11.0, t0);
   float x5 = smoothstep(16.0, 17.0, t0);
   float x6 = smoothstep(18.0, 19.0, t0);
   float x7 = smoothstep(19.0, 20.0, t0);
   float xG = smoothstep(20.0, 21.0, t0);  
   float xZ = smoothstep(21.0, 22.0, t0);
   float x8 = smoothstep(22.0, 23.0, t0);
   float x9 = smoothstep(23.0, 24.0, t0); 
   
   float x00 = smoothstep(0.0, 4.0, t0);   
   float x10 = smoothstep(4.0, 22.0, t0);
   float x11 = smoothstep(22.0, 23.0, t0);	
   float x12 = smoothstep(23.0, 24.0, t0);	

   float3 sv0 = SunDirection;
   float3 sv2 = normalize(float3(-0.09, -0.94, 0.31));
  
   float3 sv = lerp(sv2, sv2, x1);
          sv = lerp(sv, sv0, x2);
          sv = lerp(sv, sv0, x3);	  	
          sv = lerp(sv, sv0, x4);	 
          sv = lerp(sv, sv0, x5);	
          sv = lerp(sv, sv0, x6);	
          sv = lerp(sv, sv2, x7);			  
          sv = lerp(sv, sv2, x8);
          sv = lerp(sv, sv2, x9);
		  
   float dist = 0.0;	
   float distant = 0.0;	
	     distant = (height - cam_pos.z*0.1)/dir.z;
		 
   //float dth = dithering(vpos.xy, 2.0) * 2.8; 	
   float BlueNoise = bn(vpos.xy*0.3)*0.35;
   
   //float BlueNoise = bn(vpos.xy*0.04)*0.7;
   float dth = dithering(vpos.xy, 1.0) * -0.05; 
   
   float m0 = clamp(gCameraPosition.z*0.1, 300.0, 600.0);
   float r1 = (m0 - gCameraPosition.z*0.1)/dir.z;
   float r2 = 1.0;
   
   float3 rayPlane = float3(gCameraPosition.xy*0.8 + dir.xy * r1, m0);	
   float rr = lerp(0.2, 0.14, smoothstep(0.25,0.65,sv.z));

   float f = saturate(-0.044 - dot(-sv, dir.xyz));
         f = pow(f, 2.0);
         f = (f*f)/20.0;
		 f = saturate(f*7.5);

	
   float4 color = 0.2;	
    if (d0 == 1) dist = 6000.0*0.85;

   float3 lt = lerp(float3(0.03, 0.04, 0.05)*8.0, float3(0.03, 0.04, 0.05)*8.0, x1);
          lt = lerp(lt, float3(0.64, 0.6, 0.6)*0.7, x2);
          lt = lerp(lt, float3(1.0, 0.727, 0.635)*1.1, x3);
          lt = lerp(lt, float3(1.0, 0.95, 0.85)*1.0, x4);
          lt = lerp(lt, float3(1.0, 1.0, 1.0), xE);
          lt = lerp(lt, float3(1.0, 1.0, 1.0), x5);
          lt = lerp(lt, float3(1.0, 0.9, 0.8), x6);		 
          lt = lerp(lt, float3(1.0, 0.75, 0.7), x7);
		  lt = lerp(lt, float3(1.0, 0.75, 0.5)*0.1, xG);
		  lt = lerp(lt, float3(0.0275, 0.0353, 0.0471)*1.0, xZ);
          lt = lerp(lt, float3(0.0275, 0.0353, 0.0471)*4.0, x8);
          lt = lerp(lt, float3(0.0275, 0.0353, 0.0471)*6.0, x9);
          lt = lerp(pow(lt*lt, 2.5), lt, 0.1);

   float3 ShColor = float3(0.11, 0.125, 0.15);		  
   float3 ShColor2 = float3(0.051, 0.0784, 0.118)*0.65; // + float3(0.05, 0.078, 0.11)*f*3.0;
/*   
   float3 sh = lerp(ShColor2*0.7, ShColor2*0.7, x1);
          sh = lerp(sh, ShColor*2.0, x2);
          sh = lerp(sh, ShColor*4.5, x3);
          sh = lerp(sh, float3(0.380, 0.397, 0.455)*1.55, x4);
          sh = lerp(sh, float3(0.380, 0.4, 0.455)*2.0, xE);
          sh = lerp(sh, float3(0.380, 0.4, 0.455)*2.0, x5);
          sh = lerp(sh, ShColor*4.5, x6);	 
          sh = lerp(sh, ShColor*3.8, x7);
		  sh = lerp(sh, float3(0.12, 0.11, 0.14)*2.5, xG);
		  sh = lerp(sh, ShColor2*1.3, xZ);
          sh = lerp(sh, ShColor2*1.3, x8);
          sh = lerp(sh, ShColor2*1.3, x9);	
*/

float wx = currentWeather;

float cc = 1.0;

if (wx.x==2) cc = 0.0;
if (wx.x==3) cc = 0.0;
if (wx.x==4) cc = 0.0;
if (wx.x==7) cc = 0.0;
if (wx.x==8) cc = 0.0;
if (wx.x==9) cc = 0.0;
if (wx.x==12) cc = 0.0;
if (wx.x==15) cc = 0.0;
if (wx.x==16) cc = 0.0;
if (wx.x==19) cc = 0.0;


	float3 shC = lerp(ColorSkyTop.rgb, Luminance(ColorSkyTop.rgb), 1.0 - 0.5);
   float3 sh = shC*0.0032; //gFogColor.rgb		  
	      //sh+= sh*f; //12
    if (r1 >= 0.0 && r1 < dist)
	{
	    float cloudHeight;
	    float cloudHeight2;			
		float march_step = THICKNESS / STEPS,
			T = 1.0, 
			T_i = 0.0, 
			dens = 0.0, 
			alpha = 0.0,
			th_A = 1.0 / 14.69, 
			th_B_top = th_A*(1.0-0.0/25.0),
			th_B_bot = th_A*(1.0-50.0/25.0);
			
		float3 dir_step = dir / dir.z * march_step*0.75 + dth;
			
	[fastopt]
    for(int i=0; T >= 0.03 && rayPlane.z <= 600.0 && rayPlane.z >= 300.0 && (i < STEPS) && r1 - r2 < dist; i++)
		{
			rayPlane += dir_step;
            cloudHeight = saturate((rayPlane.z-600.0)/(300.0-600.0));
            cloudHeight2 = saturate((rayPlane.z-230.0)/(660.0-230.0));
            cloudHeight2 = lerp(cloudHeight2, 0.5, 1.0);			
			dens = Density(rayPlane/0.31, cloudHeight, BlueNoise) * lerp(0.84,0.9,i/STEPS);

			//density threshold
			dens*=smoothstep(lerp(th_B_bot,th_B_top,i/STEPS),th_A,dens)*march_step;

			T_i = exp(-ABSORPTION * dens);
			T *= T_i;
			
		if(dens>0.0)
		{
			float lighting = lightRay(rayPlane/0.31, sv.xyz, cloudHeight, rr, 0.02, f, BlueNoise);
		    float3 res = pow(lighting, 1.2);					
            float3 ambientLight =  sh*(cloudHeight2*100.0*2.9) +  float3(1.0, 0.9, 0.8)*res*2.0*lt*cc;  // sh*(cloudHeight2*100.0*2.9) +  float3(1.0, 0.9, 0.8)*res*12.0*lt;

			
            color.xyz+= T * exp(i/STEPS) * dens * ambientLight; //
		}				
			
			
			alpha += (1. - T_i) * (1. - alpha);
		
		}

		  //fading
		   alpha*=smoothstep(80000.0*0.36, 0.0, distant);

		   //color.xyz = 1.0; 
		   color.xyz = tonemapping(color.xyz*0.85);
		   color.xyz = pow(color.xyz, 1.0);
	       color.rgb = saturate(color.rgb);

		return float4(color.xyz, alpha);//*cc
	}
	return float4(0.0, 0.0, 0.0, 0.0);
}


float3 SunDirectionR(float GameTime)
{	
float3  f = lerp(float3(0.,0.,-.61), float3(.583,-.257,-.0108), smoothstep(0.,5.,GameTime));
        f = lerp(f, float3(.569,-.242,.151), smoothstep(5.,7.,GameTime));
		f = lerp(f, float3(.49,-.247,.323), smoothstep(7.,9.,GameTime));
		f = lerp(f, float3(.373,-.262,.445), smoothstep(9.,11.,GameTime));
		f = lerp(f, float3(.201,-.312,.517), smoothstep(11.,13.,GameTime));
		f = lerp(f, float3(-.00732,-.39,.503), smoothstep(13.,15.,GameTime));
		f = lerp(f, float3(-.189,-.502,.343), smoothstep(15.,17.,GameTime));
		f = lerp(f, float3(-.223,-.594,.1), smoothstep(17.,19.,GameTime));
		f = lerp(f, float3(0.,0.,-.61), smoothstep(19.,24.,GameTime));
		f = normalize(f);
 return f;					
}

Pixel PS_Clouds(PSInput PS, float2 vPos : VPOS0)
{
    Pixel output;

   float3 SunDirection = SunDirectionR(GameTime);  	
	
   float t0 = GameTime;
   float x1 = smoothstep(0.0, 4.0, t0);
   float x2 = smoothstep(4.0, 5.0, t0);
   float x3 = smoothstep(5.0, 6.0, t0);
   float x4 = smoothstep(6.0, 7.0, t0);
   float xE = smoothstep(8.0, 11.0, t0);
   float x5 = smoothstep(16.0, 17.0, t0);
   float x6 = smoothstep(18.0, 19.0, t0);
   float x7 = smoothstep(19.0, 20.0, t0);
   float xG = smoothstep(20.0, 21.0, t0);  
   float xZ = smoothstep(21.0, 22.0, t0);
   float x8 = smoothstep(22.0, 24.0, t0);
	
   float Timex = 0.005 * gTime;
   float Timex1 = 0.01 * gTime;	
	
   float3 TexCoord1 = normalize(PS.TexCoord1);
   float3 TexCoord2 = normalize(PS.TexCoord2);
			   
float3 sLightDir = normalize(float3(-.00732,-.39,.503));
		     
   float3 wpos = PS.WorldPos.xyz;  
   float2 res = PS.TexCoord0.xy;
   float3 p0 = (gViewInverseTranspose[3].xyz/gViewInverseTranspose[3].w);	

	
   float3 sc0 = SunLight(wpos);	
	
   float3 w1 = float3(1.0, 1.0, 10.0);
   float4 ns = float4(normalize(wpos.xyz), 1.0);    
   float ip = IntersectPlane(w1, ns.xyz);
   
   float3 ns1 = normalize(wpos.xyz);
          ns1.z = abs(ns1.z)*1.40;	

    float3 np2 = normalize(-p0+ns1);

   float4 s0 = Stars(np2 * 0.5 + 0.5)*3.2*1.40;
   		  //s0*= smoothstep(0.0, 0.3, ns.z);
		  s0.a = min(1.0, s0.a);
		  
	
   float4 cd = 1.0;
  
		  	 //c0.xyz*=wmix;

	float4 cloudSum = 1.0;
           cloudSum.a = 0.0;
	 
	float4 testt = float4(0.0,0.0,0.0,1.0); 	 
           testt = Cloud_mid(ns.xyz, gCameraPosition.xyz, 1.0, 1600.0, vPos.xy, SunDirection.xyz);				 

          cloudSum.rgb = lerp(cloudSum.rgb*testt.rgb, testt.rgb, testt.a);
          cloudSum.a = -testt.a;

          cd.xyz = cloudSum.xyz;		  
		  cd.a = min(1.0, cloudSum.a * cloudSum.a * 1.21);	
   //if (ip <= 1.0) cd = float4(0.5, 0.5, 0.5, 1.0);
                  //cd.xyz*=SunCurrentX; 
				  
    float pos = frac(vPos.xy/2.0) + 0.25;
    float3 j0 = float3(0.25,-0.25,0.25) * (1.0 / (exp2(8.0) - 1.0));
     	   j0 = lerp(2.0 * j0, -2.0 * j0, pos);					  

          cd.rgb+= j0;				  		  
	  		  
    output.World = cd;	
    output.Depth = 0.99999f;
    return output;
}

//------------------------------------------------------------------------------------------
//-- Techniques
//------------------------------------------------------------------------------------------
technique DynamicSky2tropos
{
    pass P0
    {
	    AlphaRef = 1;
        VertexShader = compile vs_3_0 VertexShaderSB();
        PixelShader = compile ps_3_0 PS_Sky();
    }
    pass P1
    {
	    AlphaRef = 1;
        VertexShader = compile vs_3_0 VertexShaderSB();
        PixelShader = compile ps_3_0 PS_Stars();
    }

    pass P2
    {
	    AlphaRef = 1;
        VertexShader = compile vs_3_0 VertexShaderSB();
        PixelShader = compile ps_3_0 PS_Moon();
    }	
	
    pass P3
    {
	    AlphaRef = 1;
        VertexShader = compile vs_3_0 VertexShaderSB();
        PixelShader = compile ps_3_0 PS_Clouds();
    }	
}
